%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB-Demo: Design of narrow-band IIR bandpass filter
% by Meinard Mueller, 20.02.2004
%
% Needed functions: es_filter.m
%
% Chebyshef filter
% Filter suggested by Eric Scheirer
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;
close all hidden;

%[file,fs,nbits] =  wavread('SYS_tonleiter1b_midi_helga.wav');
%[file,fs,nbits] =  wavread('Sys_tonleiter_CDur_langsam_BB_F.wav');
[file,fs,nbits] =  wavread('Sys_tonleiter_CDur_mittel_BB_F.wav');
%[file,fs,nbits] =  wavread('Sys_tonleiter_CDur_schnell_BB_F.wav');
%[file,fs,nbits] =  wavread('Sys_tonleiter_CDur_presto_BB_F.wav');
%[file,fs,nbits] =  wavread('Sys_tonleiter_CDur_lmsp_BB_F.wav');
%[file,fs,nbits] = wavread('BB_F_Kadenz_stacc_a.wav');
%[file,fs,nbits] = wavread('fk_bonn_44100_mono.wav');

%make "mono" from "stereo" and resample to 22050
file = file(:,1);
if fs == 44100 
    file = resample(file,1,2);
    fs = 22050;
end


%Parameters for filter design
midi = (1:128);                         % midi pitches
midi_freq = 2.^((midi-69)/12)*440;      % computing frequencies of midi pitches
nyq = fs/2;                             % Nyquist frequency
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Q = (center frequency) / bandwidth
%
% Q = 30 separates notes 
% Q=  5  does not separate notes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Q = 30;                                 
%Q = 5;

%pitch = midi_freq(57);       %A3; 
%pitch = midi_freq(60);       %C4, 261
%pitch = midi_freq(72);       %C5
pitch = midi_freq(79);       %G5
%pitch = midi_freq(96);       %C7


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Design of Chebyshef filter 
% For details use "help cheb1ord" and "help cheby1"
%
% Attenation: Construction of [b,a] may fail if the
%             filter specification are too restrictive
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pass_rel = 1/(2*Q);
stop_rel = 1/Q;
Wp = [pitch-pass_rel*pitch pitch+pass_rel*pitch]/nyq;
Ws = [pitch-stop_rel*pitch pitch+stop_rel*pitch]/nyq;
Rp = 2; 
Rs = 50;
%[n, Wn]=cheb1ord(Wp,Ws,Rp,Rs);
%[b,a]=cheby1(n,Rp,Wn);
[n, Wn]=ellipord(Wp,Ws,Rp,Rs);
[b,a]=ellip(n,Rp,Rs,Wn);
h = filter(b,a,file);
%h2 = filtfilt(b,a,file);


% pass_rel = 1/(2*Q);
% stop_rel = 1/Q;
% Wp = [pitch-pass_rel*pitch pitch+pass_rel*pitch]/nyq;
% Ws = [pitch-stop_rel*pitch pitch+stop_rel*pitch]/nyq;
% Rp = 2; 
% Rs = 4;
% %[n, Wn]=cheb1ord(Wp,Ws,Rp,Rs);
% %[b2,a2]=cheby1(n,Rp,Wn);
% [n, Wn]=ellipord(Wp,Ws,Rp,Rs);
% [b2,a2]=ellip(n,Rp,Rs,Wn);
% h2 = filter(b2,a2,file);
% %h2 = filtfilt(b2,a2,file);


pass_rel = 1/(2*Q);
stop_rel = 1/Q;
%Wp = [0.3 0.35];
%Ws = [0.25 0.4];
Wp = [0.035 0.045];
Ws = [0.03 0.05];
Rp = 2; 
Rs = 50;
%[n, Wn]=cheb1ord(Wp,Ws,Rp,Rs);
%[b,a]=cheby1(n,Rp,Wn);
[n, Wn]=ellipord(Wp,Ws,Rp,Rs);
[b,a]=ellip(n,Rp,Rs,Wn);
h = filter(b,a,file);
h2 = filtfilt(b,a,file);


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Design of  2pole-2zero-bandppass filter by direct 
% % specification of filter coefficients 
% %
% % Method suggested by Eric Scheirer (Master Thesis, 1995)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [b_es,a_es,T,BW]=es_filter(fs,pitch,Q,1);
% h_es = filter(b_es,a_es,file);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot of filtered signals and filters 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure;
subplot(3,1,1);
plot(file);
title('Signal f, h1, h2')
subplot(3,1,2);
plot(h);
subplot(3,1,3);
plot(h2);

fvtool(b,a);
%fvtool(b,a,b2,a2);
%fvtool(b,a,b_es,a_es);

%sound(file,fs);
%sound(h1,fs);
%sound(h2,fs);