function [B,A,T,BW]=es_filter(SR,CF,Q,N)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function from Eric Scheirer
%Former name: "twoptwozfilt.m" and "twoptwozparams.m"
%[B,A,T,BW] = es_filt(SR,CF,Q)    Design single bandppass filter, 2pole,2zero.
%Designs a bandpass filter with center-frequency <CF>
%(relative to a sampling rate <SR>), quality factor <Q> 
%(i.e. CF/(-3dB bandwidth) == Q) and order 2.
%This filter has one pair of complex poles, and zeros at dc and fnyq
%If N is present and greater than 1, the poles are duplicated N times.
%T is a vector of group delays in samples for each filter.
%BW is the bandwidth in Hz.
%dpwe 1995jan20 after iirbpfilt.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if(nargin < 4)
  N = 1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Former function "twoptwozparams.m"
%Calculates second-order section coefficients.
%Calculate the R and THETA for pole pairs of specified SR, CF, Q.
%N is how many times the filter is repeated - design Q is Q/N
%dpwe 1995jan24
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Qeff = Q/N;
cfrad = 2*pi*CF/SR;
%r     = 1 - cfrad/(2*Q);
% from Malcolm's SecondOrderFilter
theta = cfrad * sqrt(1 - 1/(2*Qeff)^2);
r     = exp( -cfrad/(2*Qeff) );
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


AA = [1 -2*r*cos(theta) r*r];
A = [1];
for n = 1:N
  A = conv(A,AA);
end
% figure magnitude at centre freq & normalize it
p1 = r * exp(j*theta);
p2 = conj(p1);
z  = exp(j*cfrad);

mag = abs((1-z)*(-1-z))/(abs((p1-z)*(p2-z))^N);

B = (1/mag)*[1 0 -1];

BW = CF/Q;
% T  = Q*SR/CF;	% group delay in samples
% for N = 1, envelope is max at zero
% for N = 2, IR envelope is n r^n, so max when n.log(r) = -1
% for N = 3, IR env is n.n.r^n, max when n*n*lnr = -2
T = -(N-1)/log(r);
